/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

/**
 * @file
 */

#ifndef ROBOT_ROBOT_CANBUS_H_
#define ROBOT_ROBOT_CANBUS_H_

#include <memory>
#include <string>
#include <utility>
#include <vector>
#include <stdio.h>
#include <dlfcn.h>
#include <libpcan.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include "../common/robotapp.h"
#include "ros/ros.h"
#include"candatatype.h"

/**
 * @namespace robot::canbus
 * @brief robot::canbus
 */
namespace robot {
namespace canbus {

//define mapping function according to target function in libpcan.h
typedef DWORD   (*funCAN_Init_TYPE)(HANDLE hHandle, WORD wBTR0BTR1, int nCANMsgType);
typedef HANDLE  (*funLINUX_CAN_Open_TYPE)(const char *szDeviceName, int nFlag);
typedef DWORD   (*funCAN_VersionInfo_TYPE)(HANDLE hHandle, LPSTR lpszTextBuff);
typedef DWORD   (*funCAN_Close_TYPE)(HANDLE hHandle);
typedef DWORD   (*funLINUX_CAN_Read_TYPE)(HANDLE hHandle, TPCANRdMsg* pMsgBuff);
typedef DWORD   (*funCAN_Status_TYPE)(HANDLE hHandle);
typedef DWORD   (*funCAN_Write_TYPE)(HANDLE hHandle, TPCANMsg* pMsgBuff);
typedef int     (*funnGetLastError_TYPE)(void);

//the target device name
#define DEFAULT_NODE "/dev/pcanusb32"
#define ROBOT_SUCCESS     0
#define ROBOT_FAILTURE   -1


/**
* @class Canbus
*
* @brief canbus module main class.
* It processes the control data to send protocol messages to can bus.
*/
class Canbus : public robot::common::RobotApp{
 public:
  Canbus()  {}

  /**
  * canbus总线节点名称
  */
  std::string Name() const override;

  /**
  * @brief module initialization function
  * @return initialization status
  */
  int Init() override;

  /**
  * @brief module start function
  * @return start status
  */
  int Start() override;

  /**
  * @brief module stop function
  */
  void Stop() override;

  /**
  * @brief send data to can bus
  */
  int write_data(TPCANMsg* pMsgBuff);

 private:
  // void PublishChassis();
  // void PublishChassisDetail();
  /**
  * @brief timer callback function
  */
  void OnTimer(const ros::TimerEvent &event);
  // void OnControlCommand(const apollo::control::ControlCommand &control_command);
  // apollo::common::Status OnError(const std::string &error_msg);
  // void RegisterCanClients();
  //define function pointer,there is a one-to-one mapping between target function and your defined function
  funCAN_Init_TYPE        funCAN_Init;
  funLINUX_CAN_Open_TYPE  funLINUX_CAN_Open;
  funCAN_VersionInfo_TYPE funCAN_VersionInfo;
  funCAN_Close_TYPE       funCAN_Close;
  funLINUX_CAN_Read_TYPE  funLINUX_CAN_Read;
  funCAN_Status_TYPE      funCAN_Status;
  funnGetLastError_TYPE   funnGetLastError;
  funCAN_Write_TYPE       funCAN_Write;

  /**
  * @brief can bus data read loop
  */
  int read_loop();

  /**
  * @brief read data from can bus
  */
  int read_data(TPCANMsg *pMsgBuff);

  /**
  * @brief publish data from can bus
  */
  void publish_data(TPCANMsg *pMsgBuff);

  int64_t last_timestamp_ = 0;
  ros::Timer timer_;

  const double duration = 0.01;

  const double rate = 0.005;  //s
  //GLOBALS
  void *libm_handle = NULL;//define pointer used for file acess of libpcan.so
  HANDLE pcan_handle =NULL;//void *pcan_handle
  unsigned short unBaudrate = CAN_BAUD_500K; //set the communicate baud rate of can bus
  int nExtended = CAN_INIT_TYPE_ST;       //set can message int standard model   CAN_INIT_TYPE_EX/CAN_INIT_TYPE_ST

  T_VEHICLE_STATUS_1 status1 = {0};    //VehicleStaus1
  TPCANMsg canStatus1; //VehicleStaus1 Can Message

  T_VEHICLE_STATUS_2 status2 = {0};     //VehicleStatus2
  TPCANMsg canStatus2;  //VehicleStatus2 Can Message

  T_VEHICLE_STATUS_3 status3 = {0};     //VehicleStatus3
  TPCANMsg canStatus3;  //VehicleStatus3 Can Message

  T_ICU_MONITOR icuMonitor = {0}; //ICUMonitor
  TPCANMsg canIcuMonitor; //ICUMonitor Can Message

  int heatbeat = 0;  //ICUAlive

};

}  // namespace canbus
}  // namespace robot

#endif  // ROBOT_ROBOT_CANBUS_H_
