/******************************************************************************
 * Copyright 2019 The siasun Transport Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/
#include <iterator>
#include <regex>
#include "localization.h"
#include "coordsconvertion.h"
#include <geometry_msgs/Point.h>
#include "../common/messagetopics.h"



namespace  robot{
namespace localization {
std::string Localization::Name() const { return "localization"; }

int Localization::Init() {
  LOG(INFO)<<Name()<<" init start ...";
  serial_port = "/dev/ttyUSB0";
  baudrate = 115200;
  timeout = 1000;
  node_handle_.reset(new ros::NodeHandle());
  LOG(INFO)<<Name()<<" init end.";
}

int Localization::Start() {
 LOG(INFO)<<Name()<<" start start...";
 try
 {
     //设置串口属性，并打开串口
     serial_gnss.setPort(serial_port);
     serial_gnss.setBaudrate(baudrate);
     serial::Timeout to = serial::Timeout::simpleTimeout(timeout);
     serial_gnss.setTimeout(to);
     serial_gnss.open();
 }
 catch (serial::IOException& e)
 {
     LOG(ERROR)<<Name()<< " Unable to open port " << e.what();
     return ROBOT_FAILTURE;
 }
 if (IsOpen())
 {
   GetSerialData();
 }
 LOG(INFO)<<Name()<<" start end.";
 return ROBOT_SUCCESS;
}

bool Localization::IsOpen()
{
  //检测串口是否已经打开，并给出提示信息
  if(serial_gnss.isOpen())
  {
      LOG(INFO)<<Name()<<" Serial Port initialized.";
      return true;
  }
  else
  {
      LOG(ERROR)<<Name()<< "serial Port uninitialized.";
      return false;
  }
}

int Localization::GetSerialData()
{
  try
  {
    //指定循环的频率
    ros::Rate loop_rate(50);
    while(ros::ok())
    {

      if(serial_gnss.available() >0){
          LOG(INFO)<<Name()<<"Reading from serial port\n";
          std::string strSerialResult;
          strSerialResult= serial_gnss.read(serial_gnss.available());
          LOG(INFO)<<Name()<<"Read: " << strSerialResult;
          NMEA0183 nmea;
          Analyse(strSerialResult, nmea);
          PublishData(nmea);
      }
      //处理ROS的信息，比如订阅消息,并调用回调函数
      ros::spinOnce();
      loop_rate.sleep();
    }
  }
  catch(ros::Exception ex)
  {
    LOG(ERROR)<<Name()<< " get serial data  error" << ex.what();
    return ROBOT_FAILTURE;
  }
  return ROBOT_SUCCESS;
}

void Localization::Stop() {
  serial_gnss.close();
}

int Localization::Analyse(const std::string &strSerialResult,  NMEA0183 &nmea){

    LOG(INFO)<<Name()<<"Analyse start\n";
    try
    {
        std::string delim("$");
        std::vector<std::string> v_nmeaData = Split(strSerialResult, delim);
        std::vector<std::string>::iterator iter_start = v_nmeaData.begin();
        std::vector<std::string>::iterator iter_end = v_nmeaData.end();
        std::vector<std::string>::iterator iter;
        for ( iter = iter_start; iter != iter_end; iter++)
        {
            std::string type = (*iter).substr(0, 4);
            if (type.compare("GPRMC") == 0)
            {
                GPRMC temGPRMC;
                ParaGPRMC(*iter, temGPRMC);
                nmea.gprmc = temGPRMC;
                LOG(INFO)<<Name()<<"para GPRMC end\n";
            }

            else if (type.compare("GPGGA") == 0)
            {
                GPGGA tempGPGGA;
                ParaGPGGA(*iter, tempGPGGA);
                nmea.gpgga = tempGPGGA;
                LOG(INFO)<<Name()<<"para GPGGA end\n";
            }
        }

    }
    catch (ros::Exception ex)
    {
        LOG(ERROR)<<Name()<< " Analyse  error" << ex.what();
        return ROBOT_FAILTURE;
    }
    LOG(INFO)<<Name()<<"Analyse end\n";
    return ROBOT_SUCCESS;

}


void Localization::ParaGPRMC(std::string &strGPRMC, GPRMC &gprmc){
  std::vector<std::string> nmea_gprmc = Split(strGPRMC, ",");
  gprmc.utcTime = atof(nmea_gprmc[1].c_str());
  memset(&gprmc.locationStatus, 0x00, 2);
  memcpy(&gprmc.locationStatus, nmea_gprmc[2].c_str(), 1);
  gprmc. Latitude = StrToDouble(nmea_gprmc[3].c_str(), 2);
  memset(&gprmc.LatType, 0x00, 2);
  memcpy(&gprmc.LatType, nmea_gprmc[4].c_str(), 1);
  gprmc.Longitude = StrToDouble(nmea_gprmc[5].c_str(), 3);
  memset(&gprmc.LonType, 0x00, 2);
  memcpy(&gprmc.LonType, nmea_gprmc[6].c_str(), 1);
  gprmc.Speed = atof(nmea_gprmc[7].c_str());
  gprmc.Deviated = atof(nmea_gprmc[8].c_str());
  memset(&gprmc.Date, 0x00, 5);
  memcpy(&gprmc.Date, nmea_gprmc[9].c_str(), 4);
  gprmc.Magnetic = atof(nmea_gprmc[10].c_str());
  memset(&gprmc.MagneticDirection, 0x00, 2);
  memcpy(&gprmc.MagneticDirection, nmea_gprmc[11].c_str(), 1);
  memset(&gprmc.Mode, 0x00, 2);
  memcpy(&gprmc.Mode, nmea_gprmc[12].c_str(), 1);
}

void Localization::ParaGPGGA(std::string &strGPGGA, GPGGA &gpgga){
    std::vector<std::string> nmea_gpgga = Split(strGPGGA, ",");
    gpgga.utcTime = atof(nmea_gpgga[1].c_str());
    gpgga. Latitude = StrToDouble(nmea_gpgga[2].c_str(), 2);
    memset(&gpgga.LatType, 0x00, 2);
    memcpy(&gpgga.LatType, nmea_gpgga[3].c_str(), 1);
    gpgga.Longitude = StrToDouble(nmea_gpgga[4].c_str(), 3);
    memset(&gpgga.LonType, 0x00, 2);
    memcpy(&gpgga.LonType, nmea_gpgga[5].c_str(), 1);
    memset(&gpgga.GPSStatus, 0x00, 5);
    memcpy(&gpgga.GPSStatus, nmea_gpgga[6].c_str(), 4);
    gpgga.SatelliteNumber = atoi(nmea_gpgga[7].c_str());
    gpgga.Hdop = atof(nmea_gpgga[8].c_str());
    gpgga.Height = atof(nmea_gpgga[9].c_str());
    gpgga.ReleaveHeight = atoi(nmea_gpgga[10].c_str());
    gpgga.CTKTime = atof(nmea_gpgga[11].c_str());
    gpgga.CTKID = atoi(nmea_gpgga[12].c_str());
    memset(&gpgga.Check, 0x00, 5);
    memcpy(&gpgga.Check, nmea_gpgga[13].c_str(), 4);
}


int Localization::PublishData(NMEA0183 &nmea)
{
    LOG(INFO)<<Name()<<"PublishData start\n";
    try{

        double northing, easting;
        std::string zone;
        CoordsConvertion::LLtoUTM(nmea.gprmc.Latitude, nmea.gprmc.Longitude, northing, easting, zone);
        geometry_msgs::Point utm_Point;
        utm_Point.x = easting;
        utm_Point.y = northing;
        utm_Point.z = nmea.gpgga.Height;
        ros::Publisher geometry_pub = node_handle_->advertise<geometry_msgs::Point>(robot::common::Geometry, 200);
        geometry_pub.publish(utm_Point);
    }
    catch (ros::Exception ex)
    {
        LOG(ERROR)<<Name()<< " PublishData  error" << ex.what();
        return ROBOT_FAILTURE;
    }
    LOG(INFO)<<Name()<<"PublishData end\n";
    return ROBOT_SUCCESS;

}
/* 用delim指定的正则表达式将字符串in分割，返回分割后的字符串数组 delim 分割字符串的正则表达式 */
std::vector<std::string> Localization::Split(const std::string& in, const std::string& delim) {
    std::regex re{ delim };
    // 构造函数,完成字符串分割
    return std::vector<std::string> {
        std::sregex_token_iterator(in.begin(), in.end(), re, -1),
        std::sregex_token_iterator()
    };
}

}  // namespace localization
}  // namespace robot
