/******************************************************************************
 * Copyright 2020 The siasun Transport Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 1.0 (the "License");
 *****************************************************************************/

#include <math.h>
/**
 * @file
 */

#ifndef ROBOT_ROBOT_LOCALIZATIONDATA_H_
#define ROBOT_ROBOT_LOCALIZATIONDATA_H_

/**
 * @namespace robot::localization
 * @brief robot::localization
 */
namespace robot {
namespace localization {

#define ROBOT_SUCCESS     0
#define ROBOT_FAILTURE   -1


const double RADIANS_PER_DEGREE = M_PI/180.0;
const double DEGREES_PER_RADIAN = 180.0/M_PI;

// WGS84 Parameters
const double WGS84_A = 6378137.0;		// major axis
const double WGS84_B = 6356752.31424518;	// minor axis
const double WGS84_F = 0.0033528107;		// ellipsoid flattening
const double WGS84_E = 0.0818191908;		// first eccentricity
const double WGS84_EP = 0.0820944379;		// second eccentricity

// UTM Parameters
const double UTM_K0 = 0.9996;			// scale factor
const double UTM_FE = 500000.0;		// false easting
const double UTM_FN_N = 0.0;			// false northing on north hemisphere
const double UTM_FN_S = 10000000.0;		// false northing on south hemisphere
const double UTM_E2 = (WGS84_E*WGS84_E);	// e^2
const double UTM_E4 = (UTM_E2*UTM_E2);		// e^4
const double UTM_E6 = (UTM_E4*UTM_E2);		// e^6
const double UTM_EP2 = (UTM_E2/(1-UTM_E2));	// e'^2


/**
* NMEA0183协议-GPRMC
*/
struct GPRMC
{
    // <1> UTC时间，hhmmss.ss时分秒格式
    double utcTime;
    // <2> 定位状态，A=有效定位，V=无效定位
    char locationStatus[2];
    // <3> 纬度ddmm.mmmmmmmm（度分）格式（前面的0也将被传输）
    double Latitude;
    // <4> 纬度半球N（北半球）或S（南半球）
    char LatType[2];
    // <5> 经度dddmm.mmmmmmmm（度分）格式（前面的0也将被传输）
    double Longitude;
    // <6> 经度半球E（东经）或W（西经）
    char LonType[2];
    // <7> 地面速率（000.0~999.9节，前面的0也将被传输）
    double Speed;
    // <8> 地面航向（000.0~359.9度，以真北为参考基准，前面的0也将被传输）
    double Deviated;
    // <9> UTC日期，ddmmyy（日月年）格式
    char Date[5];
    // <10> 磁偏角（000.0~180.0度，前面的0也将被传输）
    double Magnetic;
    // <11> 磁偏角方向，E（东）或W（西）
    char MagneticDirection[2];
    // <12> 模式指示（仅NMEA0183 3.00版本输出，A=自主定位，D=差分，E=估算，N=数据无效）
    char Mode[2];
};

/**
* NMEA0183协议-GPGGA
*/
struct GPGGA
{
    // 字段1：UTC 时间，hhmmss.sss，时分秒格式
    double utcTime;
    // 字段2：纬度ddmm.mmmmmmmm，度分格式（前导位数不足则补0）
    double Latitude;
    // 字段3：纬度N（北纬）或S（南纬）
    char LatType[2];
    // 字段4：经度dddmm.mmmmmmmm，度分格式（前导位数不足则补0）
    double Longitude;
    // 字段5：经度E（东经）或W（西经）
    char LonType[2];
    // 字段6：GPS状态，0=未定位，1=非差分定位，2=差分定位，3=无效PPS，6=正在估算
    char GPSStatus[2];
    // 字段7：正在使用的卫星数量（00 - 12）（前导位数不足则补0）
    int SatelliteNumber;
    // 字段8：HDOP水平精度因子（0.5 - 99.9）
    double Hdop;
    // 字段9：海拔高度（-9999.9 - 99999.9）
    double Height;
    // 字段10：地球椭球面相对大地水准面的高度
    double  ReleaveHeight;
    // 字段11：差分时间（从最近一次接收到差分信号开始的秒数，如果不是差分定位将为空）
    double CTKTime;
    // 字段12：差分站ID号0000 - 1023（前导位数不足则补0，如果不是差分定位将为空）
    int  CTKID;
    // 字段13：校验值
    char Check[5];
};

/**
* NMEA0183协议
*/
struct NMEA0183
{
    struct GPRMC gprmc;
    struct GPGGA gpgga;
};



} // localization
} // robot

#endif //ROBOT_ROBOT_LOCALIZATIONDATA_H_
